//+------------------------------------------------------------------+
//|                                                 WholeNumbers.mq5 |
//|                                        Copyright Mathieu Gilbert |
//|                                         "Widget" on ForexFactory |
//+------------------------------------------------------------------+
#property copyright "Copyright Mathieu Gilbert"
#property version "1.0"
#property indicator_chart_window
#property description "Draws major (0.0500) an minor (0.0050) whole number marks on right edge of chart window."
   
enum BarType {
   Minor = 1,
   Major = 2
};

// indicator parameters
input color LineColor = SteelBlue;
input color MinorLineColor = SteelBlue;
input color TextColor = MediumOrchid;
input ENUM_LINE_STYLE LineStyle = STYLE_DOT;
input ENUM_LINE_STYLE MinorLineStyle = STYLE_DOT;
input bool DisplayPrices = true;
input bool DisplayMinorLines = true;

// determines what a "whole number" is... major must (should) be some multiple of minor
// 1000 -> 2 digits right of most-significant digit, by 1s  (ie. 1.3400, 1.3500, 1.3600, etc.)
// 500  -> 3 digits right of most-significant digit, by 5s  (ie. 134.00, 134.50, 135.00, 135.50, etc.)
input int MinorStepSize = 500;
input int MajorStepSize = 1000;

// globals
int EXTRA_BARS = 1;                 // number of extra bars to show beyond chart window
string OBJ_NAME = "[WholeNumber]";   // each bar name prefixed with this

// used to track changes
int ChartZoom;
double ChartMax;
double ChartMin;
color ChartLineColor;
color ChartTextColor;
color ChartMinorLineColor;
ENUM_LINE_STYLE ChartLineStyle;
ENUM_LINE_STYLE ChartMinorLineStyle;
bool ChartDisplayPrices;
bool ChartDisplayMinorLines;
int ChartMinorStepSize;
int ChartMajorStepSize;


//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
void OnInit() {
   return;
}

//+------------------------------------------------------------------+
//| Custom indicator de-initialization function                      |
//+------------------------------------------------------------------+
void OnDeinit(const int reason) {
   ClearAllLines();

   return;
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate (const int rates_total,      // size of the price[] array
                 const int prev_calculated,  // bars handled on a previous call
                 const int begin,            // where the significant data start from
                 const double& price[]       // array to calculate
   ) {
   double minorStepSize = ErrorFreeDouble(MinorStepSize / MathPow(10, Digits()));
   double majorStepSize = ErrorFreeDouble(MajorStepSize / MathPow(10, Digits()));
   
   // get current visible height (vertical zoom)
   double chartMax = ChartGetDouble(0, CHART_PRICE_MAX, 0);
   double chartMin = ChartGetDouble(0, CHART_PRICE_MIN, 0);

   // get price ranges to show bars for (based on smaller step size)
   double startPrice = (MathRound(chartMin / minorStepSize) - EXTRA_BARS) * minorStepSize;
   double endPrice = (MathRound(chartMax / minorStepSize) + EXTRA_BARS) * minorStepSize;
   
   // if things changed, need to redraw
   if (NeedRedraw()) {
      ClearAllLines();
   
      for (double p = ErrorFreeDouble(startPrice); p <= endPrice; p += ErrorFreeDouble(minorStepSize)) {
         // if current price is divisible by major step size, draw major line, otherwise draw minor
         if (ErrorFreeDouble(p / majorStepSize) == MathRound(p / majorStepSize)) {
            DrawBarAt(p, Major);
         }
         else if (DisplayMinorLines) {
            DrawBarAt(p, Minor);
         }
      }
   }
   
   return(0);  
}
 

//+------------------------------------------------------------------+
//| Helper functions                                                 |
//+------------------------------------------------------------------+

// Draw a price bar at specified price level
void DrawBarAt(double price, BarType barType) {
   string barName = OBJ_NAME + DoubleToString(price);
   string textName = barName + "label";
   int zoomLevel = ChartGetInteger(0, CHART_SCALE, 0);
   int barLength = MathPow(2, 6 - zoomLevel) * PeriodSeconds();
   //int barLength = MathPow(2, 6 - zoomLevel) * 4 * Period();
   color lineColor = LineColor;
   color textColor = TextColor;
   ENUM_LINE_STYLE lineStyle = LineStyle;
   double chartShift;
   
   datetime barRight = TimeCurrent();
   datetime barLeft;
   int labelPosition = barLeft;

   // modify values if chart is shifted
   if (ChartGetInteger(0, CHART_SHIFT, 0) == true) {
      chartShift = (ChartGetDouble(0, CHART_SHIFT_SIZE, 0) / 5) * barLength;
   }
   else {
      chartShift = 4 * Period();
   }
   
      barRight = TimeCurrent() + chartShift;
      labelPosition = barRight - barLength;
   
   // modify values if minor bar
   if (barType == Minor) {
      barName = barName + "-min";
      textName = textName + "-min";
      barLength = barLength * 0.5;
      lineColor = MinorLineColor;
      lineStyle = MinorLineStyle;
   }
   
   barLeft = barRight - barLength;
   
   // create bar if doesn't exist
   if (ObjectFind(0, barName) < 0) {
      ObjectCreate(0, barName, OBJ_TREND, 0, barLeft, price, barRight, price);
      ObjectSetInteger(0, barName, OBJPROP_STYLE, lineStyle);
      ObjectSetInteger(0, barName, OBJPROP_COLOR, lineColor);
      ObjectSetInteger(0, barName, OBJPROP_RAY_RIGHT, true);
      ObjectSetInteger(0, barName, OBJPROP_BACK, 1);
   }
   
   // create label if doesn't exist
   if (ObjectFind(0, textName) < 0 && DisplayPrices && (barType == Major)) {
      ObjectCreate(0, textName, OBJ_TEXT, 0, labelPosition, price);
      ObjectSetString(0, textName, OBJPROP_TEXT, DoubleToString(price, Digits()));
      ObjectSetInteger(0, textName, OBJPROP_COLOR, textColor);
      ObjectSetInteger(0, textName, OBJPROP_BACK, 1);
   }
}

// Remove all whole number lines added to chart
void ClearAllLines() {
   string obj_Name = "";
   
   for (int i = ObjectsTotal(0, 0, -1) - 1; i >= 0 ; i--) {
      obj_Name = ObjectName(0, i, 0, -1);
      if (StringSubstr(obj_Name, 0, StringLen(OBJ_NAME)) == OBJ_NAME) {
         ObjectDelete(0, obj_Name);
      }
   }

   return;
}

// Check if something changed requiring redraw
bool NeedRedraw() {
   bool redraw = false;
  
   if (ChartLineColor != LineColor) {
      redraw = true;
      ChartLineColor = LineColor;
   }
   
   if (ChartTextColor != TextColor) {
      redraw = true;
      ChartTextColor = TextColor;
   }
   
   if (ChartLineStyle != LineStyle) {
      redraw = true;
      ChartLineStyle = LineStyle;
   }

   if (ChartMinorLineColor != MinorLineColor) {
      redraw = true;
      ChartMinorLineColor = MinorLineColor;
   }

   if (ChartMinorLineStyle != MinorLineStyle) {
      redraw = true;
      ChartMinorLineStyle = MinorLineStyle;
   }

   if (ChartDisplayPrices != DisplayPrices) {
      redraw = true;
      ChartDisplayPrices = DisplayPrices;
   }

   if (ChartDisplayMinorLines != DisplayMinorLines) {
      redraw = true;
      ChartDisplayMinorLines = DisplayMinorLines;
   }

   if (ChartZoom != ChartGetInteger(0, CHART_SCALE, 0)) {
      redraw = true;
      ChartZoom = ChartGetInteger(0, CHART_SCALE, 0);
   }
   
   if (ChartMax != ChartGetDouble(0, CHART_PRICE_MAX, 0)) {
      redraw = true;
      ChartMax = ChartGetDouble(0, CHART_PRICE_MAX, 0);
   }
   
   if (ChartMin != ChartGetDouble(0, CHART_PRICE_MIN, 0)) {
      redraw = true;
      ChartMin = ChartGetDouble(0, CHART_PRICE_MIN, 0);
   }
   
   if (ChartMinorStepSize != MinorStepSize) {
      redraw = true;
      ChartMinorStepSize = MinorStepSize;
   }
   
   if (ChartMajorStepSize != MajorStepSize) {
      redraw = true;
      ChartMajorStepSize = MajorStepSize;
   }
   
   
   return redraw;
}

// round double values to 5 decimals... floating point values get messed up
double ErrorFreeDouble(double value) {
   return MathRound(100000 * value) / 100000;
}